/** This code is free software; you can redistribute it and/or
 ** modify it under the terms of GNU Lesser General Public License.
 */

#include "common.h"
#include "iksemel.h"

#include <PA9.h>  // PAlib include
#include <netdb.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>

static void
nds_io_close (void *socket)
{
	int sock = (int) socket;
        shutdown(sock,SHUT_RDWR);
}

static int
nds_io_connect (iksparser *prs, void **socketptr, const char *server, int port)
{
	int sock = -1;

        PA_InitSocket(&sock,(char *)server,port,PA_NORMAL_TCP);
	*socketptr = (void *) sock;

	return IKS_OK;
}

static int
nds_io_send (void *socket, const char *data, size_t len)
{
	int sock = (int) socket;

	if (send (sock, data, len, 0) == -1) return IKS_NET_RWERR;
	return IKS_OK;
}

static int
nds_io_recv (void *socket, char *buffer, size_t buf_len, int timeout)
{
	int sock = (int) socket;
	fd_set fds;
	struct timeval tv, *tvptr;
	int len;

	tv.tv_sec = 0;
	tv.tv_usec = 0;

	FD_ZERO (&fds);
	FD_SET (sock, &fds);
	tv.tv_sec = timeout;
	if (timeout != -1) tvptr = &tv; else tvptr = NULL;
	if (select (sock + 1, &fds, NULL, NULL, tvptr) > 0) {
		len = recv (sock, buffer, buf_len, 0);
		if (len > 0) {
			return len;
		} else if (len <= 0) {
			return -1;
		}
	}
	return 0;
}

ikstransport iks_nds_transport = {
	IKS_TRANSPORT_V1,
	nds_io_connect,
	nds_io_send,
	nds_io_recv,
	nds_io_close,
	NULL
};
