/* Simple test home automation client.  Connects to a jabber
   server and sends a few messages to various bots, receives
   and parses a few replies.  Text-only and far from 
   fully-functional.

   20070823 Mark J Cox, mark@awe.com

   taken from bits from iksroster.c and the documentation

   I used PAlib for this, but probably don't need all
   the functions so might strip that later

   ** This code is free software; you can redistribute it and/or
   ** modify it under the terms of GNU Lesser General Public License.
*/

#include <PA9.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>

#include "iksemel.h"

extern ikstransport iks_nds_transport;

struct session {
	iksparser *prs;
	iksid *acc;
	char *pass;
	int job_done;
};

iksfilter *my_filter;
int opt_log = 0;

void j_msg(const char *fmt, ...)
{
    char buf[100];
    char *spaces = "                               ";
    static int y= 2;
    va_list args;
    va_start(args,fmt);
    vsnprintf(buf,99,fmt,args);
    PA_OutputSimpleText(1,0,y,spaces);
    if (y<19) PA_OutputSimpleText(1,0,y+1,spaces);
    if (y<18) PA_OutputSimpleText(1,0,y+2,spaces);
    PA_OutputSimpleText(1,0,y++,buf);
    if (y>20) y= 2;
    va_end(args);
}

void j_error (char *msg)
{
    j_msg("error: %s", msg);
    j_msg("HALTED");
    while(1) {
        PA_WaitForVBL();
    }
}

void client_send(struct session *sess, char *to, char *msg)
{
    iks *x = iks_make_msg (IKS_TYPE_NONE, to, msg);
    iks_send (sess->prs, x);
    iks_delete (x);
}

int on_result (struct session *sess, ikspak *pak)
{
    iks *x;

    j_msg("asking bots for status...");

    /*    x = iks_make_iq (IKS_TYPE_GET, IKS_NS_ROSTER);
    iks_insert_attrib (x, "id", "roster");
    iks_send (sess->prs, x);
    iks_delete (x);*/

    client_send(sess,"temps@drawbridge","get status");
    client_send(sess,"alarm@drawbridge","get status");
    client_send(sess,"x10@drawbridge","STATUS ON");
    client_send(sess,"cid@drawbridge","STATUS ON");

    x= iks_make_pres(IKS_SHOW_AVAILABLE,"online");
    iks_send (sess->prs, x);
    iks_delete (x);
    
    return IKS_FILTER_EAT;
}

int on_stream (struct session *sess, int type, iks *node)
{
    switch (type) {
    case IKS_NODE_START: 
      {
          iks *x = iks_make_auth (sess->acc, sess->pass, iks_find_attrib (node, "id"));
          iks_insert_attrib (x, "id", "auth");
          iks_send (sess->prs, x);
          iks_delete (x);
      }
      break;
    case IKS_NODE_NORMAL:
      {
          ikspak *pak;
          
          pak = iks_packet (node);
          iks_filter_packet (my_filter, pak);
          if (sess->job_done == 1) return IKS_HOOK;
      }
      break;
        
    case IKS_NODE_STOP:
        j_error ("server disconnected");
        
    case IKS_NODE_ERROR:
        j_error ("stream error");
    }    
    if (node) iks_delete (node);
    return IKS_OK;
}

int on_error (void *user_data, ikspak *pak)
{
    j_error ("authorization failed");
    return IKS_FILTER_EAT;
}

int on_message (struct session *sess, ikspak *pak)
{
    char *body = iks_find_cdata(pak->x,"body");
    if (!body) return IKS_FILTER_EAT;
    
    j_msg("-- message from %s",pak->from->partial);

    if (!strcmp(pak->from->user,"temps")) {
        iks *temps;
        
        temps = iks_tree(body,0,NULL);
        if (temps) {
            iks *x;
            char *y;
            
            y = iks_find_cdata(temps,"heating");
            if (y) j_msg("temps: heating=%s",y);
            y = iks_find_cdata(temps,"boost");
            if (y) j_msg("temps: boost=%s",y);
            y = iks_find_cdata(temps,"target");
            if (y) j_msg("temps: target=%s",y);
            
            x = iks_child(temps);
            while (x) {
                if (iks_type(x) == IKS_TAG &&
                    !strcmp(iks_name(x),"s")) {
                    j_msg("temps: %s=%s=%s",
                            iks_find_attrib(x,"id"),
                            iks_find_cdata(x,"n"),
                            iks_find_cdata(x,"t"));
                }
                x = iks_next(x);
            }
        }
    } else if (!strcmp(pak->from->user,"alarm")) {
        iks *alarm;
        char *newbody = (char *)iks_malloc(strlen(body)+10);
        strcpy(newbody,"<a>");   /* Seems to be bug in iks_cdata */
        strcat(newbody,body);
        strcat(newbody,"</a>");
        
        alarm = iks_tree(newbody,0,NULL);
        if (alarm) {
            char *y =  iks_find_cdata(alarm,"alarm");
            if (y) j_msg("alarm: state=%s",y);
            y = iks_find_attrib(alarm,"time");
            if (y) j_msg("alarm: time was=%s",y);
        }
        iks_free(newbody);
    } else {
        iks *x; char *newbody;
        j_msg("-- unexpected message from %s: %s",pak->from->partial,body);
        newbody = (char *)iks_malloc(strlen(body)+40);
        strcpy(newbody,"you said: ");
        strcat(newbody,body);
        x = iks_make_msg (IKS_TYPE_NONE, pak->from->partial,newbody);
        iks_send (sess->prs, x);
        iks_delete (x);
        iks_free(newbody);
    }
    return IKS_FILTER_EAT;
}

void on_log (struct session *sess, const char *data, size_t size, int is_incoming)
{
    if (is_incoming) fprintf (stderr, "RECV"); else fprintf (stderr, "SEND");
    fprintf (stderr, "[%s]", data);
}

void j_setup_filter (struct session *sess)
{
    if (my_filter) iks_filter_delete (my_filter);
    my_filter = iks_filter_new ();
    iks_filter_add_rule (my_filter, (iksFilterHook *) on_result, sess,
                         IKS_RULE_TYPE, IKS_PAK_IQ,
                         IKS_RULE_SUBTYPE, IKS_TYPE_RESULT,
                         IKS_RULE_ID, "auth",
                         IKS_RULE_DONE);
    
    iks_filter_add_rule (my_filter, (iksFilterHook *) on_message, sess,
                         IKS_RULE_TYPE, IKS_PAK_MESSAGE, IKS_RULE_DONE);
    
    iks_filter_add_rule (my_filter, on_error, sess,
                         IKS_RULE_TYPE, IKS_PAK_IQ,
                         IKS_RULE_SUBTYPE, IKS_TYPE_ERROR,
                         IKS_RULE_ID, "auth",
                         IKS_RULE_DONE);
}

void j_connect (char *jabber_id, char *pass, char *ip)
{
    struct session sess;
    int e;
    int count = 0;
    
    memset (&sess, 0, sizeof (sess));
    sess.prs = iks_stream_new (IKS_NS_CLIENT, &sess, (iksStreamHook *) on_stream);
    if (opt_log) iks_set_log_hook (sess.prs, (iksLogHook *) on_log);
    sess.acc = iks_id_new (iks_parser_stack (sess.prs), jabber_id);
    sess.pass = pass;
    
    j_setup_filter (&sess);
    j_msg("connecting to wifi...");
    PA_InitWifi();
    PA_ConnectWifiWFC();
    j_msg("connecting to server...");
    e =  iks_connect_with (sess.prs, ip,IKS_JABBER_PORT,
                           sess.acc->server, &iks_nds_transport);
    switch (e) {
    case IKS_OK:
        j_msg("logging in...");
        break;
    case IKS_NET_NODNS:
        j_error ("hostname lookup failed");
    case IKS_NET_NOCONN:
        j_error ("connection failed");
    default: 
        j_msg("error ",e);
        j_error ("io error");
    }
    
    while (1) {
        if (Pad.Newpress.Start) {
            iks *x = iks_make_pres(IKS_SHOW_UNAVAILABLE,"offline");
            j_msg("start pressed!");
            iks_send (sess.prs, x);
            iks_delete (x);
            iks_disconnect(sess.prs);
            j_error("aborted");
        }
        if (Pad.Newpress.A) {
            j_msg("a pressed!");
            client_send(&sess, "x10@drawbridge","lights living 100");
        }
        if (Pad.Newpress.B) {
            j_msg("b pressed!");
            client_send(&sess, "x10@drawbridge","lights living 0");
        }
        e = iks_recv (sess.prs, 0);
        if (e) {
            if (IKS_HOOK == e) break;
            if (IKS_NET_TLSFAIL == e) j_error ("tls handshake failed");
            if (IKS_OK != e) j_error ("io error");
        }
        if ((++count %60) == 0) {
            PA_OutputText(1,0,22,"%c2uptime %d seconds",count/60);
        }
        PA_WaitForVBL();
    }
    iks_parser_delete (sess.prs);
}

int main(void)	{
    PA_Init();
    PA_InitVBL();
    PA_InitText(1, 0);
    PA_InitText(0, 0);
    PA_SetTextCol(1, 31, 31, 31);
    PA_SetTextCol(0, 0, 0, 31);

    PA_OutputText(1,0,0,"%c1DS Home Automation Moose Client");

    j_connect ("mark@drawbridge/dsham", "mark","192.168.0.18");
    return 0;
}
